@@ -28,7 +28,7 @@
 #include "display.h"
 #include "config.h"
 
-#define WINDOW_HELP_URL "http://www.chocolate-doom.org/setup-display"
+#define WINDOW_HELP_URL "https://www.chocolate-doom.org/setup-display"
 
 extern void RestartTextscreen(void);
 
@@ -106,6 +106,7 @@ static int usegamma = 0;
 
 int graphical_startup = 1;
 int show_endoom = 1;
+int show_diskicon = 1;
 int png_screenshots = 0;
 
 // These are the last screen width/height values that were chosen by the
@@ -725,6 +726,11 @@ void BindDisplayVariables(void)
         M_BindIntVariable("show_endoom",               &show_endoom);
     }
 
+    if (gamemission == doom || gamemission == strife)
+    {
+        M_BindIntVariable("show_diskicon",             &show_diskicon);
+    }
+
     if (gamemission == heretic || gamemission == hexen || gamemission == strife)
     {
         M_BindIntVariable("graphical_startup",        &graphical_startup);
